@@ -614,6 +614,7 @@ xsmp_get_restart_style_hint (GsmClient *client);
 
 static GKeyFile *
 xsmp_save (GsmClient *client,
+           GsmApp    *app,
            GError   **error)
 {
         GsmClientRestartStyle restart_style;
@@ -672,6 +673,14 @@ xsmp_save (GsmClient *client,
                                        GSM_AUTOSTART_APP_DISCARD_KEY,
                                        exec_discard);
 
+        if (app != NULL) {
+                gsm_app_save_to_keyfile (app, keyfile, &local_error);
+
+                if (local_error) {
+                        goto out;
+                }
+        }
+
 out:
         g_free (desktop_file_path);
         g_free (exec_program);
@@ -1249,10 +1258,10 @@ save_yourself_done_callback (SmsConn   conn,
                  client->priv->description,
                  success ? "True" : "False");
 
-	if (client->priv->current_save_yourself != -1) {
-		SmsSaveComplete (client->priv->conn);
-		client->priv->current_save_yourself = -1;
-	}
+        if (client->priv->current_save_yourself != -1) {
+                SmsSaveComplete (client->priv->conn);
+                client->priv->current_save_yourself = -1;
+        }
 
         /* If success is false then the application couldn't save data. Nothing
          * the session manager can do about, though. FIXME: we could display a
